﻿/******************************************************************
 * ScheduledAgent.cs : 
 *               This is the Background Task, to be executed every 
 *               30 minutes to report this equipment trackinf information
 *               to the PandoraFMS Server.
 * 
 * Author      : Patricio Martínez León
 * Date        : September, 27th 2012
 * Modifications
 * Date         Author           Description
 * ----------------------------------------------------------------
 * 
*/

using System;
using System.Net;
using System.Net.Sockets;
using System.Device.Location;
using System.Text;
using System.Windows;
using System.Globalization;
using Microsoft.Phone.Shell;
using Microsoft.Phone.Info;
using Microsoft.Phone.Scheduler;
using Microsoft.Phone.Controls;

namespace Frameworks.GPSAgent
{
    public class ScheduledAgent : ScheduledTaskAgent
    {
        private static volatile bool _classInitialized;

        /// <summary>
        /// Message in XML format, to be sent to the PandoraFMS Server
        /// </summary>
        private string strMessage = "";

        /// <summary>
        /// This flag is true when data has been sent
        /// </summary>
        private bool dataSent = false;

        /// <summary>
        /// This flag is true when the QUIT command has been sent to the server
        /// this command finishes the TCP session using the Tentacle protocol
        /// </summary>
        private bool quitSent = false;

        /// <summary>
        /// Default name of the Agent. This is acquaired from the LINQ-To-SQL Database
        /// </summary>
        private string strAgentName = "abc";
        public string AgentName
        {
            get { return strAgentName; }
            set { strAgentName = value; }
        }

        /// <remarks>
        /// ScheduledAgent constructor, initializes the UnhandledException handler
        /// </remarks>
        public ScheduledAgent()
        {
            if (!_classInitialized)
            {
                _classInitialized = true;
                // Subscribe to the managed exception handler
                Deployment.Current.Dispatcher.BeginInvoke(delegate
                {
                    Application.Current.UnhandledException += ScheduledAgent_UnhandledException;
                });
            }
        }

        /// Code to execute on Unhandled Exceptions
        private void ScheduledAgent_UnhandledException(object sender, ApplicationUnhandledExceptionEventArgs e)
        {
            if (System.Diagnostics.Debugger.IsAttached)
            {
                // An unhandled exception has occurred; break into the debugger
                System.Diagnostics.Debugger.Break();
            }
        }

        /// <summary>
        /// Agent that runs a scheduled task
        /// </summary>
        /// <param name="task">
        /// The invoked task
        /// </param>
        /// <remarks>
        /// This method is called when a periodic or resource intensive task is invoked
        /// </remarks>
        protected override void OnInvoke(ScheduledTask task)
        {
            GeoCoordinateWatcher gcw = new GeoCoordinateWatcher(GeoPositionAccuracy.High)
            {
                MovementThreshold = 100
            };
            gcw.PositionChanged += new System.EventHandler<GeoPositionChangedEventArgs<GeoCoordinate>>(gcw_PositionChanged);
            gcw.Start(false);
        }

        /// <summary>
        /// This event is executed when the GPS detects a position change
        /// Send data to the PandoraFMS Server
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void gcw_PositionChanged(object sender, GeoPositionChangedEventArgs<GeoCoordinate> e)
        {
            AgentName = clsConfiguration.fnReadName();
            if (AgentName.Length <= 0)
                ErrorMessage("PandoraFMS", "The agent name is incorrect. Please reinstall the PandoraFMS Agent");
            GeoCoordinateWatcher gcw = (GeoCoordinateWatcher)sender;
            GeoCoordinate position = gcw.Position.Location;
            gcw.Stop();
            if (position != null)
            {
                string strPlatform = Environment.OSVersion.Platform.ToString();
                string strOsVersion = Environment.OSVersion.Version.ToString();
                strMessage = "<?xml version='1.0' encoding='utf-8' ?>\n" +
                             "<agent_data description= '' group= '' os_name='windows' os_version='" + strOsVersion +"' interval='1800' version='1.0' agent_name='" + 
                                    AgentName + "' timezone_offset='0' " +
                             "latitude='" + position.Latitude.ToString().Replace(',', '.') + "' longitude='" + position.Longitude.ToString().Replace(',', '.') + 
                             "' altitude= '" + position.Altitude.ToString().Replace(',','.') + "'>" +
                             fnModule("AplicationCurrentMemoryUsage", "Current Application Memory Usage", "generic_data", DeviceStatus.ApplicationCurrentMemoryUsage.ToString()) +
                             fnModule("AplicationPeakMemoryUsage", "Peak Memory Usage", "generic_data", DeviceStatus.ApplicationPeakMemoryUsage.ToString()) +
                             fnModule("AplicationMemoryUsageLimit", "Memory Usage Limit per Aplication", "generic_data", DeviceStatus.ApplicationMemoryUsageLimit.ToString()) +
                             fnModule("DeviceManufacturer", "Device Manufacturer", "generic_data_string", DeviceStatus.DeviceManufacturer.ToString()) +
                             fnModule("DeviceName", "Device Name", "generic_data_string", DeviceStatus.DeviceName.ToString()) +
                             fnModule("DeviceFirmwareVersion", "Device Firmware Version", "generic_data_string", DeviceStatus.DeviceFirmwareVersion.ToString()) +
                             fnModule("DeviceHardwareVersion", "Device Hardware Version", "generic_data_string", DeviceStatus.DeviceHardwareVersion.ToString()) +
                             fnModule("DeviceTotalMemory", "Device Total Memory", "generic_data", DeviceStatus.DeviceTotalMemory.ToString()) +
                             fnModule("IsKeyBoardPresent", "Is Keyboard Present in Device", "generic_proc", DeviceStatus.IsKeyboardPresent?"1":"0") +
                             fnModule("PowerSource", "Power Source", "generic_data_string", DeviceStatus.PowerSource.ToString()) +
                             fnModule("DeviceUniqueId", "Unique Id of the device", "generic_data_string", clsConfiguration.fnDeviceUniqueId()) +
                             fnModule("OsPlatform", "Operating System Platform", "generic_data_string", strPlatform) +
                             fnModule("Osversion", "Operating System Version", "generic_data_string", strOsVersion) +
                             "</agent_data>";

                IPEndPoint dirServer = new IPEndPoint(IPAddress.Parse(clsConfiguration.fnReadAddress()), 41121);
                SocketAsyncEventArgs args = new SocketAsyncEventArgs();
                args.RemoteEndPoint = dirServer;
                args.Completed += SocketAsyncEventArgs_Completed;

                Socket sck = new Socket(AddressFamily.InterNetwork,
                    SocketType.Stream,
                    ProtocolType.Tcp);
                sck.ConnectAsync(args);
            }
            else
            {
                NotifyComplete();
            }
        }

        /// <summary>
        /// Build a new MODULE tag in XML format. The information used is from one value only
        /// </summary>
        /// <param name="name">Indicator's name</param>
        /// <param name="description">Description or explanation of indicator</param>
        /// <param name="type">Data type used</param>
        /// <param name="data">Actual data</param>
        /// <returns></returns>
        string fnModule(string name, string description, string type, string data)
        {
            return "<module>" +
                             "<name><![CDATA[" + name + "]]></name>" +
                             "<description><![CDATA[" + description + "]]></description>" +
                             "<type><![CDATA[" + type + "]]></type>" +
                             "<data><![CDATA[" + data + "]]></data>" +
                             "</module>";
        }

        /// <summary>
        /// This method is executed with every asynchronous TCP event
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void SocketAsyncEventArgs_Completed(object sender, SocketAsyncEventArgs e)
        {
            // check for errors
            if (e.SocketError != SocketError.Success)
            {
                ErrorMessage("Socket: ", "Error during socket operation: " + e.SocketError);

                // do some resource cleanup
                CleanUp(e);
                return;
            }

            // check what has been executed
            switch (e.LastOperation)
            {
                case SocketAsyncOperation.Connect:
                    HandleConnect(e);
                    break;
                case SocketAsyncOperation.Send:
                    HandleSend(e);
                    break;
                case SocketAsyncOperation.Receive:
                    HandleReceive(e);
                    break;
            }
        }

        /// <summary>
        /// This event is executed when the connection to the server socket has been established
        /// </summary>
        /// <param name="e"></param>
        private void HandleConnect(SocketAsyncEventArgs e)
        {
            if (e.ConnectSocket != null)
            {
                string message = "SEND <" + AgentName + ".1" /*+ DateTime.Now.Ticks*/ + ".data> SIZE " + strMessage.Length + "\n";
                byte[] buffer = Encoding.UTF8.GetBytes(message);

                e.SetBuffer(buffer, 0, buffer.Length);
                bool completesAsynchronously = e.ConnectSocket.SendAsync(e);
                dataSent = false;
                quitSent = false;
            }
            else
            {
                ErrorMessage("Socket", "Cant connect to server");
            }
        }

        /// <summary>
        /// This event is executed when data has been sent
        /// </summary>
        /// <param name="e"></param>
        private void HandleSend(SocketAsyncEventArgs e)
        {
            if (!quitSent)
                e.ConnectSocket.ReceiveAsync(e);
            else
                // free resources
                CleanUp(e);
        }

        /// <summary>
        /// This event is executed when data reception is completed
        /// </summary>
        /// <param name="e"></param>
        private void HandleReceive(SocketAsyncEventArgs e)
        {
            if (!dataSent)
            {
                byte[] buffer = Encoding.UTF8.GetBytes(strMessage);
                e.SetBuffer(buffer, 0, buffer.Length);
                bool completesAsynchronously = e.ConnectSocket.SendAsync(e);
                dataSent = true;
            }
            else
            {
                byte[] buffer = Encoding.UTF8.GetBytes("QUIT\n");
                e.SetBuffer(buffer, 0, buffer.Length);
                bool completesAsynchronously = e.ConnectSocket.SendAsync(e);
                quitSent = true;
            }
        }

        /// <summary>
        /// This function closes the socket and notifies completion of agent actions to the OS
        /// </summary>
        /// <param name="e"></param>
        private void CleanUp(SocketAsyncEventArgs e)
        {
            if ((e.ConnectSocket != null) && (e.ConnectSocket.Connected))
            {
                e.ConnectSocket.Shutdown(SocketShutdown.Both);
                e.ConnectSocket.Close();
            }
            NotifyComplete();
        }

        /// <summary>
        /// Show an error message as a ShellToast of WP7
        /// </summary>
        /// <param name="title">Message title</param>
        /// <param name="message">Message content</param>
        private void ErrorMessage(string title, string message)
        {
            ShellToast toast = new ShellToast();
            toast.Title = title;
            toast.Content = message;
            toast.Show();
        }

        
    }
}