﻿/******************************************************************
 * clsData.cs : Data definition for the GPS Agent
 *              Includes a Configuration table used for 
 *              communication between the Agent and GUI
 * 
 * Author      : Patricio Martínez León
 * Date        : September 27th, 2012
 * Modifications
 * Date         Author           Description
 * ----------------------------------------------------------------
 * 
*/


using System;
using System.Net;
using System.Linq;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.Data.Linq;
using System.Data.Linq.Mapping;
using System.ComponentModel;
using Microsoft.Phone.Info;
using Microsoft.Phone.Data.Linq;
using Microsoft.Phone.Data.Linq.Mapping;

namespace Frameworks.GPSAgent
{
    /// <summary>
    /// Database structure definition clsData
    /// This database is stored in isolated storage
    /// This only includes one table: Configuration
    /// </summary>
    public class clsData : DataContext
    {
        static string connectionString = "DataSource=isostore:/data.sdf";
        public clsData() : base(connectionString)
        {
        }

        public Table<configuration> Configuration
        {
            get { return this.GetTable<configuration>(); }
        }

    }

    /// <summary>
    /// Fields definition of table: configuration
    /// co_name: Name of the monitored equipment
    /// co_address: IP address of the PandoraFMS server
    /// </summary>
    [Table]
    public class configuration
    {
        [Column(Name = "CO_NAME", DbType = "NVarChar(30)", IsPrimaryKey = true)]
        public String co_name { get; set; }

        [Column(Name = "CO_ADDRESS", DbType = "NVarChar(30)", IsPrimaryKey = true)]
        public String co_address { get; set; }
    }

    /// <summary>
    /// clsConfiguration: Includes only static functions to access the database through 
    /// LINQ-TO-SQL
    /// </summary>
    public class clsConfiguration
    {
        /// <summary>
        /// fnReadName: Read the equiment name stored in database.
        /// Field used is: co_name
        /// </summary>
        /// <returns>Returns the name of the registered equipment</returns>
        public static string fnReadName()
        {
            string strName = string.Empty;
            using (var db = new clsData())
            {
                if (!db.DatabaseExists())
                    strName = "";
                else
                {
                    configuration cConfig = (from p in db.Configuration
                                             select p).Single();
                    strName = cConfig.co_name;
                }
            }
            return strName;
        }

        /// <summary>
        /// fnReadAddress: Read the IP Address of the server
        /// Field used is: co_address
        /// </summary>
        /// <returns>Returns the IP address of the server to which the data is transmitted
        public static string fnReadAddress()
        {
            string strAddress = "186.69.209.162";
            using (var db = new clsData())
            {
                if (db.DatabaseExists())
                {
                    configuration cConfig = (from p in db.Configuration
                                             select p).Single();
                    strAddress = cConfig.co_address;
                }
            }
            return strAddress;
        }

        /// <summary>
        /// fnWriteData: Save the registered data by the user on registration procedure
        /// </summary>
        /// <param name="strName">This is the equipment name. This name is show in the Agents list of PandoraFMS
        /// <param name="strAddress">IP address of the server</param>
        public static void fnWriteData(string strName, string strAddress)
        {
            if (fnReadName().Length <= 0) // Check if the name exists
            {
                using (var db = new clsData())
                {
                    if (!db.DatabaseExists())
                        db.CreateDatabase();
                    db.Configuration.InsertOnSubmit(new configuration() { co_name = strName, co_address = strAddress });
                    db.SubmitChanges();
                }
            }
        }

        /// <summary>
        /// fnDeviceUniqueId: Returns the unique ID of the device
        /// </summary>
        /// <returns>Returns the unique ID of the device</returns>
        public static string fnDeviceUniqueId()
        {
            byte[] result = null;
            object uniqueId;
            if (DeviceExtendedProperties.TryGetValue("DeviceUniqueId", out uniqueId))
            {
                result = (byte[])uniqueId;
            }
            if (result != null)
                return Convert.ToBase64String(result);
            else
                return "ERROR";
        }
    }
}
