﻿/******************************************************************
 * MainPage.xaml.cs : 
 *               This is the main page of the PandoraFMS Agent for WP7
 * 
 * Author      : Patricio Martínez León
 * Date        : Septiembre 27th, 2012
 * Modifications
 * Date         Author           Description
 * ----------------------------------------------------------------
 * 
*/

using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.Device.Location;
using Microsoft.Phone.Info;
using Microsoft.Phone.Controls;
using Microsoft.Phone.Controls.Maps;
using Microsoft.Phone.Scheduler;

namespace Frameworks.PandoraFMS
{
    public partial class MainPage : PhoneApplicationPage
    {
        /// <summary>
        /// Instancia del GPS
        /// </summary>
        private readonly GeoCoordinateWatcher gcw = new GeoCoordinateWatcher(GeoPositionAccuracy.High);
        private readonly string taskName = "GPSAgent";

        /// <summary>
        /// Constructor
        /// Initialize the GPS and show the actual position on map
        /// </summary>
        public MainPage()
        {
            InitializeComponent();
            gcw.PositionChanged += new EventHandler<GeoPositionChangedEventArgs<GeoCoordinate>>(gcw_PositionChanged);
            gcw.Start(true);
            map.ZoomLevel = 12;
            map.Mode = new AerialMode(true);
        }

        /// <summary>
        /// This event is called when GPS senses change of position
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void gcw_PositionChanged(object sender, GeoPositionChangedEventArgs<GeoCoordinate> e)
        {
            position.Location = gcw.Position.Location;
            map.Center = gcw.Position.Location;

            tbLatitude.Text = gcw.Position.Location.Latitude.ToString();
            tbLongitude.Text = gcw.Position.Location.Longitude.ToString();
            tbAltitude.Text = gcw.Position.Location.Altitude.ToString();
            tbSpeed.Text = gcw.Position.Location.Speed.ToString();
        }

        /// <summary>
        /// This function is executed any time the main page is shown
        /// Get the equipment name and IP address from the isolated database
        /// Enable or disable fields. Fields are enabled only when the agent is not registered
        /// Once the agent is registered, the fields can not be changed, just make tests
        /// Finally display on screen the equipment information, this info is sent to the server by the background task
        /// </summary>
        /// <param name="e"></param>
        protected override void OnNavigatedTo(System.Windows.Navigation.NavigationEventArgs e)
        {
            string strEquipment = Frameworks.GPSAgent.clsConfiguration.fnReadName();
            if (strEquipment.Length <= 0) // No registrado
            {
                tbEquipment.Text = "";
                tbServer.Text = "186.69.209.162";
                tbEquipment.IsEnabled = true;
                tbServer.IsEnabled = true;
                btnRegister.IsEnabled = true;
                btnTest.IsEnabled = false;
            }
            else
            {
                tbEquipment.Text = strEquipment;
                tbServer.Text = Frameworks.GPSAgent.clsConfiguration.fnReadAddress();
                tbEquipment.IsEnabled = false;
                tbServer.IsEnabled = false;
                btnRegister.IsEnabled = false;
                btnTest.IsEnabled = true;
            }

            tbApplicationCurrentMemoryUsage.Text = DeviceStatus.ApplicationCurrentMemoryUsage.ToString();
            tbApplicationPeakMemoryUsage.Text = DeviceStatus.ApplicationPeakMemoryUsage.ToString();
            tbApplicationMemoryUsageLimit.Text = DeviceStatus.ApplicationMemoryUsageLimit.ToString();
            tbDeviceManufacturer.Text = DeviceStatus.DeviceManufacturer;
            tbDeviceName.Text = DeviceStatus.DeviceName;
            tbDeviceFirmwareVersion.Text = DeviceStatus.DeviceFirmwareVersion;
            tbDeviceHardwareVersion.Text = DeviceStatus.DeviceHardwareVersion;
            tbDeviceTotalMemory.Text = DeviceStatus.DeviceTotalMemory.ToString();
            tbIsKeyboardPresent.Text = DeviceStatus.IsKeyboardPresent ? "Yes" : "No";
            tbPowerSource.Text = DeviceStatus.PowerSource.ToString();
            tbDeviceUniqueId.Text = Frameworks.GPSAgent.clsConfiguration.fnDeviceUniqueId();            
        }

        /// <summary>
        /// This function is called when the Register button is pressed
        /// First of all verify if the fields have been filled with some equipment name and IP address
        /// Save data to isolated database
        /// If the Agent has been registered previously, the unregister
        /// Register the Agent to be activated as a background task
        /// Disable buttons and fields
        /// Show confirmation message as a ShellToast
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnRegister_MouseLeave(object sender, MouseEventArgs e)
        {
            if ((tbEquipment.Text.Length > 0) && (tbServer.Text.Length > 0))
            {
                Frameworks.GPSAgent.clsConfiguration.fnWriteData(tbEquipment.Text, tbServer.Text);
                PeriodicTask exist = ScheduledActionService.Find(taskName) as PeriodicTask;
                if (exist != null)
                    ScheduledActionService.Remove(taskName);
                PeriodicTask task = new PeriodicTask(taskName)
                {
                    Description = "GPS Agent for PandoraFMS"
                };
                ScheduledActionService.Add(task);
                tbEquipment.IsEnabled = false;
                tbServer.IsEnabled = false;
                btnRegister.IsEnabled = false;
                btnTest.IsEnabled = true;

                MessageBox.Show("You are now registered and your device will send your position to server every 30 minutes");
            }
            else
            {
                MessageBox.Show("Please enter Device Name and Server IP Address");
            }
        }

        /// <summary>
        /// This function is called when the Test button is pressed
        /// The message is sent after 30 seconds
        /// Display on screen a message requeting the user to quit the application
        /// NOTICE: The application must quit in order to the message be sent to the server 
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnTest_MouseLeave(object sender, MouseEventArgs e)
        {
            try
            {
                ScheduledActionService.LaunchForTest(taskName, new TimeSpan(0, 0, 30));
                MessageBox.Show("The test will be sent in 30 sec. Please quit the Agent");
            }
            catch (Exception ex)
            {
                MessageBox.Show("The agent is not registered already: " + ex.Message);
            }
        }

    }
}